@@ -17,6 +17,7 @@
 #include "scrext.h"
 
 #define SALTNAME       "[salt]"          // dummy user name of the salt
+#define RIDDLENAME     "[riddle]"        // dummy user name of the riddle
 #define ADMINNAME      "admin"           // user name of the administrator
 
 typedef struct {                         // type of structure for a record
@@ -350,8 +351,10 @@ static void dosession(TCMPOOL *mpool){
   const char *p_ummode = tcstrskipspc(tcmapget4(params, "ummode", ""));
   const char *p_umname = tcstrskipspc(tcmapget4(params, "umname", ""));
   const char *p_uminfo = tcstrskipspc(tcmapget4(params, "uminfo", ""));
-  const char *p_umpassone = tcstrskipspc(tcmapget4(params, "umpassone", "umpassone"));
-  const char *p_umpasstwo = tcstrskipspc(tcmapget4(params, "umpasstwo", "umpasstwo"));
+  const char *p_umpassone = tcstrskipspc(tcmapget4(params, "umpassone", ""));
+  const char *p_umpasstwo = tcstrskipspc(tcmapget4(params, "umpasstwo", ""));
+  const char *p_umridque = tcstrskipspc(tcmapget4(params, "umridque", ""));
+  const char *p_umridans = tcstrskipspc(tcmapget4(params, "umridans", ""));
   const char *p_fmmode = tcstrskipspc(tcmapget4(params, "fmmode", ""));
   const char *p_fmpath = tcstrskipspc(tcmapget4(params, "fmpath", ""));
   const char *p_fmname = tcstrskipspc(tcmapget4(params, "fmname", ""));
@@ -395,6 +398,9 @@ static void dosession(TCMPOOL *mpool){
   bool auth = true;
   const char *userinfo = NULL;
   const char *authcookie = NULL;
+  const char *ridque = "";
+  const char *ridans = "";
+  const char *ridcookie = NULL;
   rp = getenv("AUTH_TYPE");
   if(rp && (!tcstricmp(rp, "Basic") || !tcstricmp(rp, "Digest")) &&
      (rp = getenv("REMOTE_USER")) != NULL){
@@ -442,6 +448,19 @@ static void dosession(TCMPOOL *mpool){
         }
       }
     }
+    rp = tcmapget2(g_users, RIDDLENAME);
+    if(rp){
+      const char *pv = strstr(rp, ":");
+      if(pv){
+        ridque = tcmpoolpushptr(mpool, tcstrdup(pv + 1));
+        ridans = tcmpoolpushptr(mpool, tcmemdup(rp, pv - rp));
+      }
+    }
+    if(*ridans != '\0'){
+      const char *ridbuf = tcmapget2(params, "riddle");
+      if((ridbuf && !tcstricmp(ridbuf, ridans)) || !tcstricmp(p_umridans, ridans))
+        ridcookie = ridans;
+    }
   }
   if(!strcmp(p_act, "logout")){
     p_user = "";
@@ -451,11 +470,8 @@ static void dosession(TCMPOOL *mpool){
   }
   bool admin = auth && !strcmp(p_user, ADMINNAME);
   bool cancom = false;
-  if(!strcmp(g_commentmode, "all")){
-    cancom = true;
-  } else if(!strcmp(g_commentmode, "login") && auth){
-    cancom = true;
-  }
+  if(!strcmp(g_commentmode, "all") || (!strcmp(g_commentmode, "login") && auth) ||
+     (!strcmp(g_commentmode, "riddle") && (ridcookie || auth))) cancom = true;
   // execute the beginning script
   if(g_scrextproc){
     const char *emsg = screxterrmsg(g_scrextproc);
@@ -790,6 +806,19 @@ static void dosession(TCMPOOL *mpool){
                 tclistprintf(emsgs, "Storing the password file was failed.");
               }
             }
+          } else if(!strcmp(p_ummode, "rid")){
+            if(!checkusername(p_umridans)){
+              tclistprintf(emsgs, "The answer is invalid.");
+            } else {
+              tcmapprintf(g_users, RIDDLENAME, "%s:%s", p_umridans, p_umridque);
+              if(writepasswd()){
+                tcmapput2(vars, "chrid", p_umname);
+                ridque = p_umridque;
+                ridans = p_umridans;
+              } else {
+                tclistprintf(emsgs, "Storing the password file was failed.");
+              }
+            }
           }
         }
         TCLIST *ulist = tcmpoollistnew(mpool);
@@ -800,7 +829,7 @@ static void dosession(TCMPOOL *mpool){
           const char *pass = tcmapiterval2(name);
           if(!strcmp(name, SALTNAME)){
             salt = pass;
-          } else {
+          } else if(*name != SALTNAME[0]){
             TCMAP *user = tcmpoolpushmap(mpool, tcmapnew2(TINYBNUM));
             char *str = tcmpoolpushptr(mpool, tcstrdup(pass));
             char *pv = strchr(str, ':');
@@ -820,6 +849,8 @@ static void dosession(TCMPOOL *mpool){
         tcmapput2(vars, "view", "users");
         if(tclistnum(ulist) > 0) tcmapputlist(vars, "userlist", ulist);
         if(salt) tcmapput2(vars, "salt", salt);
+        tcmapput2(vars, "ridque", ridque);
+        tcmapput2(vars, "ridans", ridans);
       } else {
         tclistprintf(emsgs, "The user management function is not available by normal users.");
       }
@@ -895,7 +926,7 @@ static void dosession(TCMPOOL *mpool){
     }
   } else if(p_id > 0){
     // single view
-    if(!auth){
+    if(!auth && !ridcookie){
       if(mtime <= p_ifmod){
         showcache();
         return;
@@ -912,6 +943,10 @@ static void dosession(TCMPOOL *mpool){
       } else if(cancom){
         tcmapputkeep2(cols, "comnum", "0");
         tcmapputkeep2(cols, "cancom", "true");
+      } else if(!strcmp(g_commentmode, "riddle")){
+        tcmapputkeep2(cols, "comnum", "0");
+        tcmapput2(vars, "ridque", ridque);
+        tcmapput2(vars, "ridans", ridans);
       }
       const char *name = tcmapget2(cols, "name");
       if(name) tcmapput2(vars, "titletip", name);
@@ -1289,6 +1324,16 @@ static void dosession(TCMPOOL *mpool){
     if(g_sessionlife > 0) printf(" max-age=%d;", g_sessionlife);
     printf("\r\n");
   }
+  if(ridcookie){
+    printf("Set-Cookie: riddle=%s;", ridcookie);
+    if(g_sessionlife > 0) printf(" max-age=%d;", g_sessionlife);
+    printf("\r\n");
+  }
+  if(*p_comowner != '\0' && checkusername(p_comowner)){
+    printf("Set-Cookie: comowner=%s;", p_comowner);
+    if(g_sessionlife > 0) printf(" max-age=%d;", g_sessionlife);
+    printf("\r\n");
+  }
   printf("\r\n");
   fwrite(tmplstr, 1, strlen(tmplstr), stdout);
   fflush(stdout);
